"Use strict";
import powerbi from "powerbi-visuals-api";
import IVisualHost = powerbi.extensibility.visual.IVisualHost;
import DataView = powerbi.DataView;
import * as visualInterfaces from "../visual/visualInterfaces";
import * as d3 from "d3";
import { IFilterManager } from "../visual/visualInterfaces";
import * as models from 'powerbi-models';
import { select, json } from "d3";

export class FilterManager implements IFilterManager{
    private target: models.IFilterColumnTarget={table:'',column:''};
    private host:IVisualHost;
    private _jsonFilter:models.IAdvancedFilter;
    
    public get jsonFilter() : models.IAdvancedFilter {
        return this._jsonFilter;
    }
    public get advancedEditModeMark():string{
        return 'SyinpoSlicer_AdvancedEditMode_'+this.inAdvancedEditMode;
    }
    constructor(host:IVisualHost){
        this.host=host;
    }
    _inAdvancedEditMode: boolean=false;
    public set inAdvancedEditMode(newValue:boolean){
        this._inAdvancedEditMode=this._inAdvancedEditMode?true:newValue;
    }
    public get inAdvancedEditMode():boolean{
        return this._inAdvancedEditMode;
    }
    setFilter(jsonFilter:models.IAdvancedFilter,applyNow:boolean): void {
        jsonFilter.conditions[0].value=this.advancedEditModeMark
        this._jsonFilter=jsonFilter;
        if(applyNow){
            this.applyJsonFilter();
        }
    }

    public applyJsonFilter():void{
        console.debug('filter manager applyJsonFilter start');
        this.host.applyJsonFilter(this._jsonFilter,'general','filter',powerbi.FilterAction.merge);
        console.debug(this.jsonFilter);
        console.debug('filter manager applyJsonFilter end');

    }
    public dispose():void{
        console.debug('filterManager disposing');

        if(this.target.table&&this.target.column){
            this.clear();
        }
        console.debug('filterManager disposed');

    }
    public clear():void{
        console.debug("clear start:");
        this.host.applyJsonFilter(null,"general","filter",powerbi.FilterAction.remove);
        console.debug("clear end");
    }
    public update(field:powerbi.DataViewCategoryColumn):void{
        let queryName:string=field.source.queryName;
        let splitPosition:number=queryName.indexOf('.');
        this.target.table=queryName.substring(0,splitPosition); // table
        this.target.column=queryName.substring(splitPosition+1,queryName.length); // column
        console.debug("queryName:",queryName);
        console.debug("queryName.length:",queryName.length);
        console.debug("filterManager update end, now filter:",this.target);
    }
    public setFilter_String(selection:string[],applyNow:boolean):void{
        console.debug("setFilter_String start:",selection);
        let conditions:models.IAdvancedFilterCondition[]=[];

        conditions.push({
            "operator":"StartsWith",
            "value":this.advancedEditModeMark
        });

        selection.forEach(function(item){
            conditions.push({
                "operator":"Is",
                "value":item
            });
        });
        let jsonFilter:models.IAdvancedFilter={
            "$schema": "http://powerbi.com/product/schema#advanced",
            "target": this.target,
            "filterType": models.FilterType.Advanced,
            "conditions":conditions,
            "logicalOperator":"Or"
        }
        console.debug("jsonFilter:",jsonFilter);
        this._jsonFilter=jsonFilter;
        if(applyNow){
            this.applyJsonFilter();
        }
        console.debug("setFilter_String end");
    }
}