import { Selector } from "./selector";
import * as settings from "../../settings";
import powerbi from "powerbi-visuals-api";
import * as d3 from "d3";
import { selection } from "d3";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
import {pixelConverter} from "powerbi-visuals-utils-typeutils";

export class DropDownSelector extends Selector {
    public select(items: string[]): void {
        console.debug('dropDownSelector select items:',items);
        this.dropDown.selectAll('option').each(function(d,i){
            console.debug('this.text:',d3.select(this).text());
            if(items.includes(d3.select(this).text())){
                console.debug('dropDownSelector select, included this:',this);
                d3.select(this).attr('selected','selected');
            }else{
                console.debug('dropDownSelector select, not included this:',this);
                d3.select(this).attr('selected',null);
            }
        });
    }
    public updateFormat(itemsSetting: settings.ItemsSetting, width: number, height: number) {
        console.debug('dropDownSelector updateFormat start');
        if (itemsSetting) {
            this.dropDown.style('background-color', itemsSetting.backgroundColor)
                .attr('postion','absolute')
                .style('color', itemsSetting.fontColor)
                .style('font-size',pixelConverter.fromPointToPixel(itemsSetting.textSize) + 'px')
                .style('width', (width - 10) + 'px')
                .style('border-style', settings.Enums.getOutlineStyle(itemsSetting.outline))
                .style('border-width', '2px')
                .style('border-color', 'black');
            this.dropDown.selectAll('option').style('background-color', itemsSetting.backgroundColor)
                .style('color', itemsSetting.fontColor)
                .style('font-size', pixelConverter.fromPointToPixel(itemsSetting.textSize) + 'px')
                .style('border-style', settings.Enums.getOutlineStyle(itemsSetting.outline))
                .style('border-width', '2px')
                .style('border-color', 'black');
        }
        console.debug('dropDownSelector updateFormat end');
    }
    protected checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn): boolean {
        //The defaultSelect is nullable, so first check the new defaultSelect
        console.debug('checkDefaultSelectionChange start');
        let flag: boolean;
        if (!defaultSelect || !defaultSelect.values || !defaultSelect.values[0]) {
            console.debug('new defaultSelect is null, set flag=false')
            flag = flag || false;
        } else {
            console.debug('new defaultSelect is not null, check the oldDefaultSelect');
            console.debug('new defaultSelect:', defaultSelect);
            if (!this.defaultSelect || !this.defaultSelect.values || !defaultSelect.values[0]) {
                console.debug('previous defaultSelect is null, set flag=true');
                flag = flag || true;
            } else {
                if (defaultSelect.values[0].toString() != this.defaultSelect.values[0].toString()) {
                    console.debug('new defaultSelect not equal to the previous, set flag=true');
                    flag = flag || true;
                } else {
                    console.debug('new defaultSelect equal to the previous, set flag=false');
                    flag = flag || false;
                }
            }
        }
        console.debug('checkDefaultSelectionChange end, result:', flag);
        return flag;
    }
    private container: Selection<HTMLDivElement>;
    private dropDown: Selection<HTMLSelectElement>;
    protected createView(container: Selection<HTMLDivElement>) {
        console.debug('dropDownViewManager', 'createView start');
        this.container = container;
        this.dropDown = this.container.append("select").classed("dropDown-selector", true).classed("selector", true);//.attr('multiple',true);
        let dropDownSelector=this;
        //on change, filter
        this.dropDown.on("input change", function () {
            console.debug('DropDownSelector:', "input change");
            console.debug('this', this);
            let selectedValues: string[] = [];
            for (let i = 0; i < this.selectedOptions.length; i++) {
                let option = this.selectedOptions.item(i);
                if (option.selected) {
                    console.debug("option selected", option);
                    console.debug('option.text:', option.text);
                    selectedValues.push(option.text);
                }
            }
            if (selectedValues.length == 0) {
            } else {
                console.debug("selection", selectedValues);
                dropDownSelector.emit(dropDownSelector.stringFieldFilter_event,selectedValues,true);
            }
            console.debug("this", this.selectedOptions);
        });

        console.debug('dropDownViewManager', 'createView end');

    }
    public dispose() {
        console.debug('dropDown-selector disposing');
        this.dropDown.remove();
        console.debug('dropDown-selector disposed');

    }
    public update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('dropDownViewManager', 'update start');
        //Check field, defaultSelect are changed or not



        let needUpdateDefaultSelection: boolean = this.checkFieldAndDefaultSelectionChange(field, defaultSelect, defaultStart, defaultEnd);
        let newDefaultSelect: string;
        if (needUpdateDefaultSelection && defaultSelect && defaultSelect.values && defaultSelect.values[0]) {
            newDefaultSelect = defaultSelect.values[0].toString();
            console.debug('newDefaultSelect:', newDefaultSelect);
        }
        //let values=field.values
        let options: d3.Selection<HTMLOptionElement, powerbi.PrimitiveValue, HTMLSelectElement, any> = <d3.Selection<HTMLOptionElement, powerbi.PrimitiveValue, HTMLSelectElement, any>>this.dropDown.selectAll("option").data(field.values, function (d) { return d.toString(); });//map data
        options.exit().remove();//delete

        if (needUpdateDefaultSelection && newDefaultSelect) {
            console.debug('reset defaultSelection start');
            options = options.enter().append("option")
                .text(function (d) { return d.toString(); })
                .attr('label', function (d) { return d.toString(); })
                .classed('dropDown-option', true)
                .attr('title',function (d) { return d.toString(); })
                .merge(options)
                .attr("selected", function (d) {
                    console.debug('d.toString():', d.toString());
                    console.debug('newDefaultSelect:', newDefaultSelect);
                    return (d.toString() == newDefaultSelect) ? 'selected' : null;
                });
            console.debug('Set defaultSelection end');
        } else {
            console.debug('defaultSelection not reset, start update options');
            options=options.enter().append("option")
                .text(function (d) { return d.toString(); })
                .attr('label', function (d) { return d.toString(); })
                .attr('title',function (d) { return d.toString(); })
                .attr('label', function (d) { return d.toString(); }).classed('dropDown-option', true).merge(options);//add
            console.debug('defaultSelection not reset, end update options');
        }

        let selectedValues: string[] = [];
        this.dropDown.selectAll('option').each(function(d,i){
            let option=d3.select(this);
            console.debug('option',option.text());
            console.debug('selected?',option.property('selected'));
            if(option.property('selected')){
                console.debug('find a selected option:',option.text());
                selectedValues.push(option.text());
            }
        });
        if (selectedValues.length == 0) {
        } else {
            console.debug("selection", selectedValues);
        }
        this.emit(this.stringFieldFilter_event,selectedValues,false);
        console.debug("dropDown:", this.dropDown);
        this.field = field;
        this.defaultSelect = defaultSelect;
        console.debug('new defaultSelect:', defaultSelect);
        console.debug('this.defaultSelect=defaultSelect, this.defaultSelect:', this.defaultSelect);
        console.debug('dropDownViewManager', 'update end');
    }
}