import powerbi from "powerbi-visuals-api";
import * as settings from "../../settings";
import * as d3 from "d3";
import { EventEmitter } from "events";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
export interface ISelector extends EventEmitter{
    update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):void;
    dispose():void;
    select(items:string[]):void;
    updateFormat(itemsSetting:settings.ItemsSetting,width:number,height:number):void;
    stringFieldFilter_event:string;//filter a string field, give one param: string[]
}

export abstract class Selector extends EventEmitter implements ISelector{
    protected field: powerbi.DataViewCategoryColumn;
    protected defaultSelect: powerbi.DataViewValueColumn;
    protected defaultStart: powerbi.DataViewValueColumn;
    protected defaultEnd: powerbi.DataViewValueColumn;
    //events
    public stringFieldFilter_event:string="stringFieldFilter_event";

    protected checkFieldChange(field: powerbi.DataViewCategoryColumn):boolean{
        //Field is not null, so first check the old field
        if(!this.field){
            return true;
        }
        if(field.source.queryName==this.field.source.queryName){
            console.debug('checkFieldChange: false');
            return false;
        }else{
            console.debug('checkFieldChange: true');
            return true;
        }
    }
    protected abstract checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):boolean;
    protected checkFieldAndDefaultSelectionChange(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){
        console.debug('checkFieldAndDefaultSelectionChange start');
        let flag:boolean=this.checkFieldChange(field)||this.checkDefaultSelectionChange(defaultSelect,defaultStart,defaultEnd)
        console.debug('checkFieldAndDefaultSelectionChange end, result:',flag);
        return flag;
    }
    constructor(container:Selection<HTMLDivElement>){
        super();
        console.debug("Abstract selector:","constructor start");
        this.createView(container);
    }
    public abstract select(items: string[]):void;
    public abstract updateFormat(itemsSetting: settings.ItemsSetting,width:number,height:number):void;
    protected abstract createView(container:Selection<HTMLDivElement>):void;
    public abstract dispose():void;
    public abstract update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):void;
}