"Use strict";
import powerbi from "powerbi-visuals-api";
import DataView = powerbi.DataView;
import IVisualHost = powerbi.extensibility.visual.IVisualHost;
import {VisualSettings} from "../settings/visualSettings";
import * as d3 from "d3";
import { IFilter } from "powerbi-models";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
import {EventEmitter} from "events";
import * as models from 'powerbi-models';

/*
Main interfaces
*/

//IVisualSettings


//ISelectorManager
export interface ILayoutManager{
    layout:Selection<HTMLElement>;
    update(dataView:DataView,width:number,height:number):void;
    dispose():void;
}
export interface ILayoutManagerConstructor{
    new(container:Selection<HTMLElement>,filterManager:IFilterManager):ILayoutManager;
}

//ISelectorManager

export interface ISelectorManager{
    selectorContainer:Selection<HTMLElement>;
    stringFieldFilter_event:string;//filter a string field, give one param: string[]
    dispose():void;
    updateData(field:powerbi.DataViewCategoryColumn,defaultSelect:powerbi.DataViewValueColumn,defaultStart:powerbi.DataViewValueColumn,defaultEnd:powerbi.DataViewValueColumn):void;
    updateFormat(visualSettings:VisualSettings,width:number,height:number):void;
    select(items:string[]):void;
    
}
export interface ISelectorManagerConstructor{
    new(selectorContainer:Selection<HTMLElement>):ISelectorManager&EventEmitter;
}

//IFilterManager
export interface IFilterManager{
    jsonFilter:models.IAdvancedFilter;
    inAdvancedEditMode:boolean;
    update(field:powerbi.DataViewCategoryColumn):void;
    setFilter_String(selection:string[],applyNow:boolean):void;
    setFilter(jsonFilter:models.IAdvancedFilter,applyNow:boolean):void;
    applyJsonFilter():void;
    clear():void;
    dispose():void;
}
export interface IFilterManagerConstructor{
    new(host:IVisualHost):IFilterManager;
}
